#! /usr/bin/perl
use strict;
use warnings;
#///////////////////////////////////////////////////////////////////
#/								   /
#/   Control_Floor.pl                                              /
#/                                                                 /
#/   A program to control the temperature of the tile floor        /
#/   in the kitchen and dining room of Bill Farmer's house.        /
#/                                                                 /
#/   The floor was equiped with 12 T-probe sensors below the tile  /
#/   before the tile was laid.  There is also electric resistance  /
#/   heater wiring in the sub floor below the tile.  This program  /
#/   uses the AUX signal of the LinkUSB to turn on/off a pair of   /
#/   solid state relays (240 volts).                               /
#/							           /
#/   This program executes on the Sheeva Plug in the basement      /
#/   at address 192.168.2.102					   /
#/								   /
#/   To execute this program: sudo perl Control_Floor.pl xx.x	   /
#/   where xx.x is the desired temperature in degrees F.	   /
#/   If the setpoint is not supplied in the command line, a	   /
#/   default setpoint of 72.0 will be used.			   /
#/								   /
#/   This program runs continously with a poll time of 11 seconds  /
#/								   /
#/   This program implements a proportional heater control with    /
#/   a control band of setpoint-1.0 to setpoint.		   /
#/								   /
#/   The proportional heater control has a range of 0-10seconds on /
#/   which relates to 0-100% error in the control band.		   /
#/								   /
#/   Copyright 2010 ... iButtonLink LLC                            /
#/   All rights reserved					   /
#/   N8921 Stone School Rd.					   /
#/   East Troy, Wi. 53120                                          /
#/   United States of America                                      /
#/   Phone (262)662-4029					   /
#/   Fax:  (262)436-2403					   /
#/                                                                 /
#/   Kindly direct comments, etc to support@iButtonLink.com        /
#/								   /
#///////////////////////////////////////////////////////////////////
use ibl;                        # The iButtonLink subroutine package
my  $Tscale = "F";              # Report temperature in American
my  $IBmaster;                  # Object from IBL.pm
my  $cm;                        # A place to store the channel mask
my  $ok;
my  $IBLdev;
my  $x;
my  $perr;			# Proportional controller error in percent
my  $thetime;			# TimeStamp
my  %HSN;
my  $hub = 0;                   # If 1, the IBL master is a LinkHub
my  @y;				# Scratchpad
my  ($key,$value,$result);
my  $heaterstate = 0;		# Start with heater off
my  $setpoint = shift || 72.0;	# Get the set point or default to 72.0
my  $avg = 0;			# Average of all the temp sensors
$IBmaster = new IBL;

$IBmaster->device('/dev/ttyUSB0');      # The LinkUSB serial device
$IBmaster->interface('Serial');         # The protocol is Serial ASCII
$ok = $IBmaster->open();

$x = $IBmaster->master;
chop $x;
print "The master is: $x\nThe setpoint is $setpoint\n";

print "Heater Off\n";
$IBmaster->write("z");			# Aux low does not echo
%HSN = $IBmaster->ClassifyedInventory;  # Get a hash of classified sensors
#/
#/
#/
while (1) {
   #///////////////////////////////////////////////////////
   #/   Read each of the sensors and accumulate average   /
   #///////////////////////////////////////////////////////
   $IBmaster->ConvertT();
   $x = 0; $avg = 0;
   while (($key, $value) = each(%HSN)) {
      $result = $IBmaster->Read_Sensor($key, $value);
      @y = split(/,/,$result);
      $avg += $y[1];			# Add temp reading to accumulator
      $x++;
      $thetime = $IBmaster->Get_the_time;
      print "$thetime,$key,$result\n";
   }
   $avg = $avg/$x;			# Calculate average temperature
   if ($avg < $setpoint) {
      if ($avg > ($setpoint-1.0)) {	# If we are in the control band
         $perr = abs(($avg-$setpoint)*100);
#         print "Proportional error $perr\n";
         Heater($perr);
      } else {
	 Heater(100.0);
      }
      $heaterstate = 1;			# Remember heater state
   }
   if ($avg >= $setpoint) {
      if ($heaterstate == 1) {
        $thetime = $IBmaster->Get_the_time;
#	print "$thetime, Heater Off\n";
      }
      Heater(0);			# Heater on for 0%
      $heaterstate=0;			# Remember heater state
   }
}
exit 0;

#/////////////////////////////////////////////////////////
#/							 /
#/   Heater ... This routine takes a parameter of 0-100  /
#/              and turns the heater on for 0-10 secs.   /
#/							 /
#/   This implement a proportional heater with a power   /
#/   setting from 0 - 100%.				 /
#/							 /
#/////////////////////////////////////////////////////////
sub Heater {
   use Time::HiRes qw (sleep);	# Overload so sleep takes a floating number
   my $hon = shift;		# Get the percentage seconds heater is on
   $hon *= 0.1;			# Calculate seconds
#   print "Heater on $hon seconds\n";
   my $hoff = 10-$hon;	        # Compute number of seconds heater is off
   $IBmaster->write('d') if $hon > 0;	# Turn the heater on if time > 0
   sleep ($hon);
   $IBmaster->write('z');	# Turn the heater off
   sleep ($hoff);
   return;
}
